// -*- mode: c++ -*- 
/* 

    GIFT, a flexible content based image retrieval system.
    Copyright (C) 1998, 1999, 2000 CUI, University of Geneva

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "libMRML/include/mrml_const.h"
#include "libMRML/include/my_throw.h"
#include "libGIFTAcHtDig/include/CAcHtDig.h"
#include <assert.h>
#include "libMRML/include/GIFTExceptions.h"
#include <fstream>
#include "libMRML/include/CXMLElement.h"
#include "libMRML/include/CAttributeList.h"

CAcHtDig::CAcHtDig(const CXMLElement& inCollectionElement){
  mMutexHtDig.lock();
  pair<bool,string> lCollectionID(inCollectionElement.stringReadAttribute(mrml_const::collection_id));
  pair<bool,string> lConfigFileLocation(inCollectionElement.stringReadAttribute("cui-htdig-config-location"));

  mIsWellConstructed=lConfigFileLocation.first;
  if(mIsWellConstructed){
    mConfigFileLocation=lConfigFileLocation.second;
  }else{
    mConfigFileLocation=
      string("The config file location was not specified. Please check your gift-config.mrml\n"
	     "The collection in question was ")
      + (lCollectionID.first?string("Not given either"):lCollectionID.second);
  }

  pair<bool,string> lHtSearchLocation(inCollectionElement.stringReadAttribute("cui-htsearch-location"));
  mIsWellConstructed = mIsWellConstructed && lHtSearchLocation.first;
  if(mIsWellConstructed){
    mHtSearchLocation=lHtSearchLocation.second;
  }else{
    mHtSearchLocation=
      string("The htsearch executable location was not specified. Please check your gift-config.mrml\n"
	     "The collection in question was ")
      + (lCollectionID.first
	 ?string("Not given either")
	 : lCollectionID.second);
  }

  mMutexHtDig.unlock();
}

CAcHtDig::operator bool()const{
  return mIsWellConstructed;
}
/**
   give back the location of the htdig config file
*/
const string& CAcHtDig::getConfigFileLocation()const{
    return mConfigFileLocation;
};
/**
   give back the location of the htdig config file
*/
const string& CAcHtDig::getHtSearchLocation()const{
    return mHtSearchLocation;
};
